"""
ustcwlt
v1.0
url.py
生成不同请求的url
"""

__all__ = ["generate_url"]


def generate_url(urltype, data=None):
    """
    urltype: "base"       data:  None
          "showc"             None
          "setc"              (port, time) int(1-9) int(sec)
          "fastsetc"          None
          "showp"             None
          "setp"              (port, time)
          "logout"            None
    """
    url = "http://wlt.ustc.edu.cn/cgi-bin/ip"
    if urltype == "base":
        return url

    if urltype == "showc":
        url += "?cmd=disp"
        return url

    if urltype == "setc":
        url += "?cmd=set&url=URL&urltype=" + str(data[0] - 1) + \
                      "&exp=" + str(data[1])
        return url

    if urltype == "fastsetc":
        url += "?cmd=set"
        return url

    if urltype == "showp":
        url += "?cmd=pref"
        return url

    if urltype == "setp":
        url += "?cmd=pref&url=URL&urltype=" + str(data[0] - 1) + \
                      "&exp=" + str(data[1])
        return url

    if urltype == "logout":
        url += "?cmd=logout"
        return url



"""
ustcwlt
v1.0
account.py
WltAccount类，提供所有功能
"""


__all__ = ["WltAccount"]

import urllib
import urllib.parse
import urllib.error
import urllib.request
import http.cookiejar
from ustcwlt.error import *
from ustcwlt.html import *
from ustcwlt.url import *


class WltAccount:
    def __init__(self, user_name: str, password: str):
        """
        param user_name: str 网络通账号的用户名
        param password:  str 账号密码
        """
        try:
            req = urllib.request.Request(generate_url("base"))
            response = urllib.request.urlopen(req)
            html = response.read().decode("GBK")
        except urllib.error.URLError:
            raise NetworkError

        self.is_login = False
        self.access = True
        self.user_name = str(user_name)
        self.password = str(password)
        self.ip = analyse_html(html)["ip"]

        cookie = http.cookiejar.CookieJar()
        handler = urllib.request.HTTPCookieProcessor(cookie)
        self.opener = urllib.request.build_opener(handler)

    def login(self, connect: bool = False):
        """
        登录账号
        param connect: bool 默认为False
                            若为False，仅登录账号
                            若为True，登录账号并根据常用设置开通网络
        """
        login_data = {
            "cmd":          "login",
            "url":          "URL",
            "ip":           self.ip,
            "name":         self.user_name,
            "password":     self.password,
        }
        if connect:
            login_data["set"] = " "
        else:
            login_data["go"] = " "

        login_data = urllib.parse.urlencode(login_data).encode("GBK")

        try:
            req = urllib.request.Request(generate_url("base"), login_data)
            response = self.opener.open(req)
            html = response.read().decode("GBK")
        except urllib.error.URLError:
            raise NetworkError

        info = analyse_html(html)
        if info["urltype"] == "failed":
            raise LoginError(info["msg"])

        self.is_login = True

        self.access = info["access"]
        if connect and (not self.access):
            raise AccessError("您没有使用网络通对外连接的权限")

    def get_info(self) -> dict:
        """
        获取账号信息
        return: dict
        "ip":           str 当前网络的IP地址
        "currentport":  int 当前账号的网络出口
        "prefport":     int 常用设置的网络出口
        "preftime":     int 常用设置的连接时间（秒）；若为0，则代表永久
        "access":       int 若为0, 则当前账号未开通网络通服务；若为1，则已开通
        """
        if not self.is_login:
            raise AccessError("未登录无法进行操作")

        try:
            req = urllib.request.Request(generate_url("showc"))
            response = self.opener.open(req)
            htmlc = response.read().decode("GBK")
            req = urllib.request.Request(generate_url("showp"))
            response = self.opener.open(req)
            htmlp = response.read().decode("GBK")
        except urllib.error.URLError:
            raise NetworkError

        infoc = analyse_html(htmlc)
        infop = analyse_html(htmlp)

        return {
            "ip":              infoc["ip"],
            "currentport":     infoc["port"],
            "prefport":        infop["port"],
            "preftime":        infop["time"],
            "access":          self.access
            }

    def set_connection(self, port: int = None, time: int = None):
        """
        开通网络
        param port: int 网络出口
        param time: int 开通时间（秒）
        特别，当port和time都为None（默认情况）时，按照常用设置开通网络
        """
        if self.access == 0:
            raise AccessError("您没有使用网络通对外连接的权限")
        if not self.is_login:
            raise AccessError("未登录无法进行操作")
        if not ((port is None) and (time is None)):
            if type(port) != int:
                raise TypeError("参数port应为int类型")

            if type(time) != int:
                raise TypeError("参数time应为int类型")

            if port <= 0 or port >= 10:
                raise ValueError("port超出范围")

            if time < 0:
                raise ValueError("time超出范围")

            connection_url = generate_url("setc", (port, time))
        else:
            connection_url = generate_url("fastsetc")

        try:
            req = urllib.request.Request(connection_url, method="GET")
            self.opener.open(req)
        except urllib.error.URLError:
            raise NetworkError

    def set_preference(self, port: int, time: int):
        """
        更改常用设置
        param port: int 网络出口
        param time: int 开通时间（秒）
        """
        if not self.is_login:
            raise AccessError("未登录无法进行操作")

        if type(port) != int:
            raise TypeError("参数port应为int类型")

        if type(time) != int:
            raise TypeError("参数time应为int类型")

        if port <= 0 or port >= 10:
            raise ValueError("port超出范围")

        if time < 0:
            raise ValueError("time超出范围")

        preference_url = generate_url("setp", (port, time))

        try:
            req = urllib.request.Request(preference_url, method="GET")
            self.opener.open(req)
        except urllib.error.URLError:
            raise NetworkError

    def logout(self):
        """
        退出登录
        """
        if not self.is_login:
            raise AccessError("未登录无法进行操作")

        logout_url = generate_url("logout")
        try:
            req = urllib.request.Request(logout_url, method="GET")
            self.opener.open(req)
        except urllib.error.URLError:
            raise NetworkError
        self.is_login = False



"""
ustcwlt
v1.0
一个提供南七技校校园网（网络通）登录、查询、设置功能的Python库
"""

import ustcwlt.account
import ustcwlt.error

WltAccount = ustcwlt.account.WltAccount
NetworkError = ustcwlt.error.NetworkError
AccessError = ustcwlt.error.AccessError
LoginError = ustcwlt.error.LoginError



"""
ustcwlt
v1.0
html.py
使用正则表达式解析html页面
"""


__all__ = ["analyse_html"]


import re


def analyse_html(html):
    """
    return：

    "urltype"         "ip"         "port"          "msg"         "time"      "access"

    login          [ip]
    failed                                      [msg]
    info           [ip]      [currentport]                                 0 or 1
    pref                    [prefport] int 1-9           [preftime] int sec
     
    """
    if re.search("公用计算机", html):
        ip = re.search("([1-9][0-9]*\.[1-9][0-9]*\.[1-9][0-9]*\.[1-9][0-9]*)", html).group(1)
        return {"urltype": "login", "ip": ip}

    if re.search("请重新登录", html):
        msg = re.search("信息：<br>(.*)<p>", html).group(1)
        return {"urltype": "failed", "msg": msg}

    if re.search("校内测速", html):
        currentport = int(re.search("出口: ([1-9])", html).group(1))
        ip = \
        re.search("当前IP地址([1-9][0-9]*\.[1-9][0-9]*\.[1-9][0-9]*\.[1-9][0-9]*)", html).group(1)

        access = 1
        if re.search("您没有使用网络通对外连接的权限", html):
            access = 0

        return {"urltype": "info", "port": currentport, "ip": ip, "access": access}

    if re.search("返回主界面", html):
        prefport = int(re.search("常用出口：([1-9])", html).group(1))
        if re.search("常用时间：永久", html):
            preftime = 0
        else:
            rst = re.search("常用时间：([1-9]*)([小分秒])", html)
            preftime = int(rst.group(1))
            if rst.group(2) == "小":
                preftime *= 3600
            elif rst.group(2) == "分":
                preftime *= 60
        return {"urltype": "pref", "port": prefport, "time": preftime}


"""
ustcwlt
v1.0
error.py
自定义异常类
"""

__all__ = ["NetworkError", "AccessError", "LoginError"]


class NetworkError(Exception):
    """
    网络连接出错
    在创建WltAccount类的实例及调用各方法时都可能触发
    """
    def __init__(self, err="网络连接出错"):
        Exception.__init__(self, err)


class AccessError(Exception):
    """
    无权进行操作
    在调用get_info(), set_connection(), set_preference()方法时，
    若还未登录或账户未开用网络通服务，会抛出此异常
    """
    def __init__(self, err="无权进行操作"):
        Exception.__init__(self, err)


class LoginError(Exception):
    """
    登录出现问题
    在调用login()方法时，若使用非科大IP地址，输入的用户名不存在，
    或输入的密码错误，会抛出此异常
    """
    def __init__(self, err="登录出现问题"):
        Exception.__init__(self, err)